unit ser_comm;
{ Serielle Kommunikation ber das WIN-API durchfhren.
  Es kann Com1 bis Com9 verwendet werden und nur eine
  Schnittstelle, da nur ein Handle verwaltet wird. }

interface

{ Typen fr Empfangspuffer: }
type Pser_ebuff=^ser_ebuff;
     ser_ebuff=array[0..255] of char;

var ser_comm_emsg:Longword; { ev. Fehlermeldungen }
    ser_comm_estr:string; { Fehlermeldung }
    ser_aktiv:boolean;
          { Liefert True, wenn auf den COM-Port zugegriffen werden darf. }
    beende_app:boolean;   { Sollte das Hauptprogramm setzen, wenn das
                            Programm beendet werden soll }
    ser_Dein:ser_ebuff;  { Empfangspuffer }

{ COM-Port ffnen: }
function ser_oeffne(cport:byte; baudr:Cardinal; wl,par,sb:byte):integer;
          { cport: 1..9 ... Port; baudr: 600..256000; wl: 4..8, Wortlnge;
            par:0,1,2 mit 0..keine, 1..ungerade, 2..gerade, 3..mark, 4..space;
            sb: 0..2 Stoppbits mit 0..1, 1..1.5, 2..2;
            Ergebnis:  0 ... alles o.k.
                    -1 ... ungltige Parameter
                    -2 ... Handle bereits belegt
                    -3 ... Schnittstelle nicht frei
                    -4 ... DeviceControlBlock kann nicht geladen werden
                    -5 ... DeviceControlBlock kann nicht geschrieben werden }


{ String senden: }
function ser_sends(dat:string):integer;
          { Es wird versucht den String dat komplett zu senden.
            Rckgabewert:
                     -1 ... COM-Port nicht geffnet
                     sonst: Anzahl der gesendeten Bytes. Ist diese kleiner
                            als len(dat), wird die WIN-API-Fehlernummer
                            in ser_comm_emsg bergeben. }


{ String empfangen: }
function ser_gets(anz:Cardinal):integer;
          { Es wird versucht anz<256 Bytes  zu empfangen und in ser_Dein
            abzulegen.
            Rckgabewert:
                     -1 ... COM-Port nicht geffnet
                     sonst: Anzahl der empfangenen Bytes. Ist diese kleiner
                            als anz, wird die WIN-API-Fehlernummer
                            in ser_comm_emsg bergeben. }

{ COM-Port schlieen: }
function ser_beende:Integer;

{ TimeOut setzen: }
function ser_setto(RIT,RTTM,RTTC,WTTM,WTTC:Cardinal):Integer;
          { !!! Parameter werden nicht berprft!!!
           Parameter in ms entsprechend Win32-Programmers-HB.
           Ist RIT=0, dann ergibt sich das Timeout zum Lesen aus:
           RTTM*(Anzahl an Bytes)+RTTC, zum Schreiben genauso mit
           WTTM und WTTC.
           Rckgabewerte:
                0 ... hat geklappt,
               -1 ... Port nicht geffnet
               -2 ... Fehlermeldung von WIN-API, Fehlernummer in
                                    ser_comm_emsg }

{ DTR auf 1 und RTS auf 0 setzen: }
function ser_setdtrrts(inv:boolean):Integer;
          { Manche Gerte wie z.B. METEX-Multimeter versorgen Ihr
            Interface ber diese Leitungen, die entsprechend gesetzt
            werden mssen.
            !!! Damit ist kein Hardware-Handshake mglich!!!
            inv=False: DTR=1 und RTS=0
            inv=True:  DTR=0 und RTS=1
           Rckgabewerte:
                0 ... hat geklappt,
               -1 ... Port nicht geffnet
               -2 ... Fehlermeldung von WIN-API, Fehlernummer in
                                    ser_comm_emsg }
{ Handshake-Verfahren setzen: }
{ function ser_seths(modus:byte):integer;   ... fehlt noch }

{ Testweise: }
function ser_getflags:Integer;
         { Flags des DCB zurckgeben }

implementation
uses windows, SysUtils; { WIN-API-Deklarationen }

var ok:boolean;
    ser_handle:Thandle;


function ser_oeffne(cport:byte; baudr:Cardinal; wl,par,sb:byte):integer;

    var ok,antw:boolean; fname:String;
    tmp:integer; ser_c_dcb:Tdcb;

begin ok:=true; 
{ Welcher Port ? }
      if cport in [1..9] then fname:='COM'+char(cport+48)
         else begin ok:=false;
                    ser_oeffne:=-1; end;
      if ok then
{ Schnittstelle ffnen: }
      if ser_handle=0 then begin
          ser_handle:=CreateFile(PChar(fname),(Generic_Read or Generic_Write),
          0,nil,Open_Existing,0,0);
          if ser_handle=invalid_handle_value then begin
             ok:=false; ser_oeffne:=-3;
             ser_comm_emsg:=GetLastError;
             ser_handle:=0; end; end
      else begin ok:=false; ser_oeffne:=-2; end;
{ Parameter laden, berprfen und neu setzen: }
      if ok then
         antw:=GetCommState(ser_handle,ser_c_dcb);
      if not(antw) then begin ok:=false; ser_oeffne:=-4; end;
      if ok then begin
{ Baudrate: umrechnen in Integer um ordinalen Typ zu erhalten:}
         if baudr mod 8 = 0 then begin
           tmp:=baudr div 8;
           case tmp of
             75,150,300,600,1200,1800,2400,4800,7000,7200,
                   14400,16000,32000: ser_c_dcb.Baudrate:=baudr;
           else begin ok:=false; ser_oeffne:=-1; end; end; end
         else begin ok:=false; ser_oeffne:=-1; end; end;
{ Wortlnge: }
      if ok then
         if wl in [4..8] then ser_c_dcb.ByteSize:=wl
            else begin ok:=false; ser_oeffne:=-1; end;
{ Paritt: }
      if ok then
          if par in [0,1,2] then ser_c_dcb.parity:=par
             else begin ok:=false; ser_oeffne:=-1; end;
{ Stoppbits: 0=1, 1=1.5, 2=2, 1 funktioniert wohl nicht }
      if ok then
         if sb in [0,2] then ser_c_dcb.stopbits:=Byte(sb)
            else begin ok:=false; ser_oeffne:=-1; end;
{ Flags als default auf 0 setzen }
        ser_c_dcb.Flags:=LongInt(0);
      if ok then begin
         antw:=SetCommState(ser_handle,ser_c_dcb);
         if antw then ser_oeffne:=0
                else begin ser_oeffne:=-5;
                ser_comm_emsg:=GetLastError;
                ser_comm_estr:=SysErrorMessage(ser_comm_emsg); end; end;
{ Alles lschen was noch im Buffer sein knnte: }
      if ok then 
         antw:=PurgeComm(ser_handle,purge_txabort or purge_rxabort or
                         purge_txclear or purge_rxclear);
      if ok then ser_aktiv:=true;
end;



function ser_sends(dat:string):integer;
var ok:boolean; laenge,anz:Cardinal;
begin if ser_handle<>0 then begin
         laenge:=length(dat);
         ok:=WriteFile(ser_handle,Pchar (dat)^,laenge,anz,nil);
         ser_sends:=anz;
         if not ok then ser_sends:=anz; ser_comm_emsg:=GetLastError;
         end
       else begin ser_sends:=-1; ser_aktiv:=false; end;
end;


function ser_gets(anz:Cardinal):integer;
var ok:boolean; laenge:Cardinal; Pein:Pser_ebuff;
begin if ser_handle<>0 then begin
        laenge:=0;
        ok:=ReadFile(ser_handle,ser_Dein,anz,laenge,nil);
        ser_gets:=laenge;
        if not ok then ser_comm_emsg:=GetLastError; end
      else begin ser_gets:=-1; ser_aktiv:=false; end;
end;


function ser_beende:integer;
var ok:boolean;
begin if ser_handle<>0 then begin
         ser_aktiv:=false;
{         ok:=CancelIO(ser_handle);  NUR WIN-NT }
         ok:=CloseHandle(ser_handle);
         ser_handle:=0; ser_beende:=0; end
      else ser_beende:=-1;
end;


function ser_setto(RIT,RTTM,RTTC,WTTM,WTTC:Cardinal):Integer;

var tob:CommTimeouts; ok:boolean;

begin if ser_handle<>0 then begin
        with tob do begin
           ReadIntervalTimeout:=RIT;
           ReadTotalTimeoutMultiplier:=RTTM;
           ReadTotalTimeoutConstant:=RTTC;
           WriteTotalTimeoutMultiplier:=WTTM;
           WriteTotalTimeoutConstant:=WTTC; end;
        ok:=SetCommTimeouts(ser_handle,tob);
        if ok then ser_setto:=0
        else begin ser_setto:=-2;
                 ser_comm_emsg:=GetLastError; end;
        end
      else begin ser_setto:=-1; ser_aktiv:=false; end;
end;


function ser_setdtrrts(inv:boolean):Integer;
var ser_c_dcb:_dcb; antw:boolean;
begin if ser_handle<>0 then begin
        antw:=GetCommState(ser_handle,ser_c_dcb);
        if inv then ser_c_dcb.flags:=4097
           else ser_c_dcb.Flags:=17;
        antw:=SetCommState(ser_handle,ser_c_dcb);
        if antw then ser_setdtrrts:=0
           else begin ser_setdtrrts:=-2;
               ser_comm_emsg:=GetLastError; end; end
      else begin ser_setdtrrts:=-1; ser_aktiv:=false; end;
end;

function ser_getflags:Integer;
var ser_c_dcb:_dcb; antw:boolean;

begin antw:=GetCommState(ser_handle,ser_c_dcb);
      ser_getflags:=ser_c_dcb.Flags;
end;


initialization ser_handle:=0; beende_app:=false; ser_aktiv:=false;

finalization if ser_handle<>0 then ok:=CloseHandle(ser_handle);

end.
